
from sqlalchemy.ext.declarative import declarative_base

from sqlalchemy.schema import MetaData

metadata = MetaData(schema="pl_inbox")
DeclarativeBase = declarative_base(metadata=metadata)


def loadStorageTuples():
    from . import Task
    Task.__unused = False

    from . import TaskAction
    TaskAction.__unused = False

    from . import Activity
    Activity.__unused = False

    from . import Setting
    Setting.__unused = False



import logging
from datetime import datetime

from peek_plugin_inbox._private.PluginNames import inboxTuplePrefix
from vortex.Tuple import Tuple, addTupleType, TupleField

logger = logging.getLogger(__name__)


@addTupleType
class ActivityTuple(Tuple):
    """ Activity Tuple

    An Activity represents an item in the activity screen
    This is a screen that is intended to show actions that have been performed recently
        or events that plugins want in this list.
    
    see InboxAbiABC.NewActivity for documentation.
        
    """
    __tupleType__ = inboxTuplePrefix + 'ActivityTuple'

    pluginName: str = TupleField()
    uniqueId: str = TupleField()
    userId: str = TupleField()
    dateTime: datetime = TupleField()

    # The display properties of the task
    title: str = TupleField()
    description: str = TupleField()
    iconPath: str = TupleField()

    # The mobile-app route to open when this task is selected
    routePath: str = TupleField()
    routeParamJson: str = TupleField()

    # Auto Delete on Time
    autoDeleteDateTime: datetime = TupleField()



"""Added Task.displayPriority

Peek Plugin Database Migration Script

Revision ID: da155db8b8f1
Revises: 297359a078eb
Create Date: 2017-04-16 19:18:25.347113

"""

# revision identifiers, used by Alembic.
revision = 'da155db8b8f1'
down_revision = '297359a078eb'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('Task', sa.Column('displayPriority', sa.Integer(), server_default='0', nullable=True), schema='pl_inbox')
    op.execute('UPDATE "pl_inbox"."Task" set "displayPriority" = 0;')
    op.alter_column('Task', 'displayPriority', schema='pl_inbox', nullable=False, type_=sa.Integer())
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('Task', 'displayPriority', schema='pl_inbox')
    # ### end Alembic commands ###



"""Added pluginName

Peek Plugin Database Migration Script

Revision ID: 7e6bd0411082
Revises: deab93942032
Create Date: 2018-12-03 01:01:08.855992

"""

# revision identifiers, used by Alembic.
revision = '7e6bd0411082'
down_revision = 'deab93942032'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    op.execute('TRUNCATE TABLE pl_inbox."Task" CASCADE ')
    op.execute('TRUNCATE TABLE pl_inbox."Activity" CASCADE ')

    op.drop_constraint('Activity_uniqueId_key', 'Activity', schema='pl_inbox', type_='unique')
    op.add_column('Activity', sa.Column('pluginName', sa.String(), nullable=False), schema='pl_inbox')
    op.create_index('idx_Activity_pluginName_uniqueId', 'Activity', ['pluginName', 'uniqueId'], unique=True, schema='pl_inbox')

    op.drop_constraint('Task_uniqueId_key', 'Task', schema='pl_inbox', type_='unique')
    op.add_column('Task', sa.Column('pluginName', sa.String(), nullable=False), schema='pl_inbox')
    op.create_index('idx_Task_pluginName_uniqueId', 'Task', ['pluginName', 'uniqueId'], unique=True, schema='pl_inbox')


def downgrade():
    op.drop_index('idx_Task_pluginName_uniqueId', table_name='Task', schema='pl_inbox')
    op.drop_column('Task', 'pluginName', schema='pl_inbox')
    op.create_unique_constraint('Task_uniqueId_key', 'Task', ['uniqueId'], schema='pl_inbox')

    op.drop_index('idx_Activity_pluginName_uniqueId', table_name='Activity', schema='pl_inbox')
    op.drop_column('Activity', 'pluginName', schema='pl_inbox')
    op.create_unique_constraint('Activity_uniqueId_key', 'Activity', ['uniqueId'], schema='pl_inbox')



from typing import Type

from peek_plugin_base.client.PluginClientEntryHookABC import PluginClientEntryHookABC
from peek_plugin_base.server.PluginServerEntryHookABC import PluginServerEntryHookABC

__version__ = '2.5.5'


def peekServerEntryHook() -> Type[PluginServerEntryHookABC]:
    from peek_plugin_inbox._private.server.PluginServerEntryHook import \
        PluginServerEntryHook
    return PluginServerEntryHook


def peekClientEntryHook() -> Type[PluginClientEntryHookABC]:
    from peek_plugin_inbox._private.client.PluginClientEntryHook import \
        PluginClientEntryHook
    return PluginClientEntryHook




from abc import ABCMeta, abstractmethod
from datetime import datetime
from typing import Optional, List

import pytz
from twisted.internet.defer import Deferred


class NewTask:
    """ TaskTuple

    A TaskTuple represents the feature rich mechanism for notifications, alerts and messages
     sent from initiator plugins to mobile devices.

    """

    # Auto complete options, Must match _private.storage.Task.Task.state
    AUTO_COMPLETE_OFF = 0
    AUTO_COMPLETE_ON_DELIVER = 1
    AUTO_COMPLETE_ON_SELECT = 2
    AUTO_COMPLETE_ON_ACTION = 4
    AUTO_COMPLETE_ON_DIALOG = 16

    # Auto delete options, Must match _private.storage.Task.Task.state
    AUTO_DELETE_OFF = 0
    AUTO_DELETE_ON_DELIVER = 1
    AUTO_DELETE_ON_SELECT = 2
    AUTO_DELETE_ON_ACTION = 4
    AUTO_DELETE_ON_COMPLETE = 8
    AUTO_DELETE_ON_DIALOG = 16

    # notification mask (multiple options allowed)
    NOTIFY_BY_DEVICE_POPUP = 1
    NOTIFY_BY_DEVICE_SOUND = 2
    NOTIFY_BY_SMS = 4
    NOTIFY_BY_EMAIL = 8
    NOTIFY_BY_DEVICE_DIALOG = 16

    # Display options
    DISPLAY_AS_TASK = 0
    DISPLAY_AS_MESSAGE = 1

    # Priority options
    PRIORITY_SUCCESS = 1
    PRIORITY_INFO = 2
    PRIORITY_WARNING = 3
    PRIORITY_DANGER = 4

    def __init__(self, pluginName: str, uniqueId: str, userId: str, title: str,
                 description: Optional[str] = None, iconPath: Optional[str] = None,
                 displayAs: int = DISPLAY_AS_TASK,
                 displayPriority: int = PRIORITY_SUCCESS,
                 routePath: Optional[str] = None, routeParamJson: Optional[dict] = None,
                 autoComplete: int = AUTO_COMPLETE_OFF,
                 autoDelete: int = AUTO_DELETE_OFF,
                 autoDeleteDateTime: Optional[datetime] = None,
                 onDeliveredPayloadEnvelope: Optional[bytes] = None,
                 onCompletedPayloadEnvelope: Optional[bytes] = None,
                 onDeletedPayloadEnvelope: Optional[bytes] = None,
                 onDialogConfirmPayloadEnvelope: Optional[bytes] = None,
                 notificationRequiredFlags: int = 0,
                 actions: List['NewTaskAction'] = (),
                 overwriteExisting=False):
        """
        :param pluginName: The name of the plugin creating this activity.

        :param uniqueId: A unique identifier provided when this task was created.
            The initiating plugin may use this later to cancel the task.
            HINT : Ensure you prefix the uniqueId with your plugin name.
    
        :param userId: A string representing the unique ID of the user. This must match the
            users plugin.
    
        :param title: The title to display in the task.
        :param description: The long text that is displayed under the title for this task.
        :param iconPath: The URL for the icon, if any.
        :param displayAs: Should this task be displayed as a message or task?
        :param displayPriority: What priority should the task display as?
    
        :param routePath: If this route path is defined, then selecting the task
            will cause the peek client fe to change routes to a new page.
        :param routeParamJson: If the route path is defined, this route param json 
            will be passed along when the route is switched..
    
        :param autoComplete: Should this task auto complete?
                This parameter defines what state it will auto complete in.
                See the AUTO_COMPLETE... class constants
        :param autoDelete: Should this task auto delete?
                This parameter defines what state it will auto delete in.
                See the AUTO_DELETE... class constants
        :param autoDeleteDateTime: The datetime when this task should automatically
                be deleted it if still exists.
    
        :param onDeliveredPayloadEnvelope: (Optional) The payload that will be delivered locally
            on Peek Server when the task is delivered.
        :param onCompletedPayloadEnvelope: (Optional) The payload that will be delivered locally
            on Peek Server when the task is completed (auto, or otherwise)
        :param onDeletedPayloadEnvelope: (Optional) The payload that will be delivered locally
            on Peek Server when the task is deleted (auto, or otherwise)
        :param onDialogConfirmPayloadEnvelope: (Optional) The payload that will be delivered
            locally on Peek Server when the user clicks "OK" on the dialog.
            
        :param overwriteExisting: If a task with that uniqueId already exists, it will be
            deleted.
        """
        self.pluginName = self._required(pluginName, "pluginName")
        self.uniqueId = self._required(uniqueId, "uniqueId")
        self.userId = self._required(userId, "userId")

        # The display properties of the task
        self.title = self._required(title, "title")
        self.description = description
        self.iconPath = iconPath

        self.displayAs = displayAs
        self.displayPriority = displayPriority
        if not self.displayPriority in (1, 2, 3, 4):
            raise Exception("Invalid displayPriority %s" % self.displayPriority)

        # The mobile-app route to open when this task is selected
        self.routePath = routePath
        self.routeParamJson = routeParamJson

        # The confirmation options
        self.onDeliveredPayloadEnvelope = onDeliveredPayloadEnvelope
        self.onCompletedPayloadEnvelope = onCompletedPayloadEnvelope
        self.onDeletedPayloadEnvelope = onDeletedPayloadEnvelope
        self.onDialogConfirmPayloadEnvelope = onDialogConfirmPayloadEnvelope

        self.autoComplete = autoComplete
        self.autoDelete = autoDelete
        self.autoDeleteDateTime = autoDeleteDateTime
        self.overwriteExisting = overwriteExisting

        self.notificationRequiredFlags = notificationRequiredFlags

        # The actions for this TaskTuple.
        self.actions = list(actions)

    def _required(self, val, desc):
        if not val:
            raise Exception("%s is not optional" % desc)

        return val


class NewTaskAction:
    """ TaskTuple Action

    This object represents the TaskTuple Actions.
    Tasks have zero or more actions that can be performed by the user when they
    receive a task.

    """

    def __init__(self, title: str, confirmMessage: str,
                 onActionPayloadEnvelope: Optional[bytes] = None):
        """
        :param title: The title of the action, this will appear as a menu option.
        :param confirmMessage: This is the message that will be shown to confirm the action.
        :param onActionPayloadEnvelope: This payload will be delivered locally on Peek Server
                 When the action is performed on the user device.
        """
        self.title = self._required(title, "title")
        self.confirmMessage = self._required(confirmMessage, "confirmMessage")
        self.onActionPayloadEnvelope = self._required(onActionPayloadEnvelope,
                                                      "onActionPayload")

    def _required(self, val, desc):
        if not val:
            raise Exception("%s is not optional" % desc)

        return val


class NewActivity:
    """ TaskTuple

    A TaskTuple represents the feature rich mechanism for notifications, alerts and messages
     sent from initiator plugins to mobile devices.

    """

    def __init__(self, pluginName: str, uniqueId: str, userId: str, title: str,
                 autoDeleteDateTime: datetime,
                 dateTime: Optional[datetime] = None,
                 description: Optional[str] = None, iconPath: Optional[str] = None,
                 routePath: Optional[str] = None, routeParamJson: Optional[dict] = None,
                 overwriteExisting=False):
        """

        :param pluginName: The name of the plugin creating this activity.
        
        :param uniqueId: A unique identifier provided when this activity was created.
            The initiating plugin may use this later to cancel the activity.
            HINT : Ensure you prefix the uniqueId with your plugin name.
    
        :param userId: A string representing the unique ID of the user. This must match the
            users plugin.
    
        :param title: The title to display in the activity.
        :param description: The long text that is displayed under the title for this activity.
        :param iconPath: The URL for the icon, if any.
    
        :param routePath: If this route path is defined, then selecting the activity
            will cause the peek client fe to change routes to a new page.
        :param routeParamJson: If the route path is defined, this route param json 
            will be passed along when the route is swtiched.
            
        :param autoDeleteDateTime: The time and date when this activity will be deleted.
            
        :param overwriteExisting: If an activity with that uniqueId already exists,
            it will be deleted.
        
        """
        self.pluginName = self._required(pluginName, "pluginName")
        self.uniqueId = self._required(uniqueId, "uniqueId")
        self.userId = self._required(userId, "userId")
        self.dateTime = dateTime if dateTime else datetime.now(pytz.utc)

        # The display properties of the task
        self.title = self._required(title, "title")
        self.description = description
        self.iconPath = iconPath

        # The mobile-app route to open when this item is selected
        self.routePath = routePath
        self.routeParamJson = routeParamJson

        # Auto Delete on Time
        self.autoDeleteDateTime = autoDeleteDateTime

        self.overwriteExisting = overwriteExisting

    def _required(self, val, desc):
        if not val:
            raise Exception("%s is not optional" % desc)

        return val


class InboxApiABC(metaclass=ABCMeta):
    @abstractmethod
    def addTask(self, task: NewTask) -> Deferred:
        """ Add a New Task

        Add a new task to the users device.
        
        :param task: The definition of the task to add.
        :return :code:`Deferred` firing with None
        
        """

    @abstractmethod
    def completeTask(self, pluginName: str, uniqueId: str) -> Deferred:
        """ Complete a Task
        
        Mark a task as complete. NOTE, This doesn't delete it.

        :param pluginName: The plugin that this task is for.
        :param uniqueId: The uniqueId provided when the task was created.
        :return :code:`Deferred` firing with None

        """

    @abstractmethod
    def getTasks(self, pluginName: str, uniqueIdLike: Optional[str] = None,
                 userId: Optional[str] = None) -> Deferred:
        """ Get Tasks

        Retrieve a list of tasks matching the criteria

        :param pluginName: The plugin that this task is for.
        :param userId: The userId of the task.
        :param uniqueIdLike: The uniqueId provided when the task was created,
            this is queried with the SQL ilike expression %
        :return :code:`Deferred` firing with a List[TaskTuple]

        """

    @abstractmethod
    def removeTask(self, pluginName: str, uniqueId: str) -> Deferred:
        """ Remove a Task
        
        Remove a task from the users device.

        :param pluginName: The plugin that this task is for.
        :param uniqueId: The uniqueId provided when the task was created.
        :return :code:`Deferred` firing with None

        """

    @abstractmethod
    def removeTasks(self, pluginName: str, uniqueIdLike: Optional[str] = None,
                    userId: Optional[str]=None) -> Deferred:
        """ Remove Tasks

        Remove all tasks created by a plugin.

        :param pluginName: The plugin that these tasks are for.
        :param uniqueIdLike: The uniqueId provided when the task was created,
            this is queried with the SQL ilike expression %
        :param userId: The userId of the tasks to remove.
        :return :code:`Deferred` firing with None

        """

    @abstractmethod
    def addActivity(self, activity: NewActivity) -> Deferred:
        """ Add a new Activity item

        Add a new Activity to the users device.

        :param activity: The definition of the activity to add.
        :return :code:`Deferred` firing with None

        """

    @abstractmethod
    def removeActivity(self, pluginName: str, uniqueId: str) -> Deferred:
        """ Remove an Activity item

        Remove an Activity from the users device.

        :param pluginName: The plugin that this activity is for.
        :param uniqueId: The uniqueId provided when the activity was created.
        :return :code:`Deferred` firing with None

        """

    @abstractmethod
    def removeActivities(self, pluginName: str, uniqueIdLike: Optional[str] = None,
                         userId: Optional[str]=None) -> Deferred:
        """ Remove Tasks

        Remove all activities created by a plugin.

        :param pluginName: The plugin that these activities are for.
        :param uniqueIdLike: The uniqueId provided when the task was created,
            this is queried with the SQL ilike expression %
        :param userId: The userId of the activities to remove.
        :return :code:`Deferred` firing with None

        """

    @abstractmethod
    def getActivities(self, pluginName: str, uniqueIdLike: Optional[str] = None,
                      userId: Optional[str] = None) -> Deferred:
        """ Get Activities

        Retrieve a list of activities matching the criteria

        :param pluginName: The plugin that this activity is for.
        :param userId: The userId of the task.
        :param uniqueIdLike: The uniqueId provided when the task was created,
            this is queried with the SQL ilike expression %
        :return :code:`Deferred` firing with a List[ActivityTuple]

        """



"""Removed all tables

Peek Plugin Database Migration Script

Revision ID: 3e75fb851c5a
Revises: 36a9bc546606
Create Date: 2017-10-30 23:34:22.804094

"""

# revision identifiers, used by Alembic.
revision = '3e75fb851c5a'
down_revision = '36a9bc546606'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2
from sqlalchemy.dialects import postgresql

def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('SettingProperty', schema='pl_inbox')
    op.drop_table('TaskAction', schema='pl_inbox')
    op.drop_table('Activity', schema='pl_inbox')
    op.drop_table('Setting', schema='pl_inbox')
    op.drop_table('Task', schema='pl_inbox')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('Task',
    sa.Column('id', sa.INTEGER(), server_default=sa.text('nextval(\'pl_inbox."Task_id_seq"\'::regclass)'), nullable=False),
    sa.Column('uniqueId', sa.VARCHAR(length=100), autoincrement=False, nullable=False),
    sa.Column('userId', sa.VARCHAR(length=50), autoincrement=False, nullable=False),
    sa.Column('dateTime', postgresql.TIMESTAMP(), autoincrement=False, nullable=False),
    sa.Column('title', sa.VARCHAR(length=100), autoincrement=False, nullable=False),
    sa.Column('description', sa.VARCHAR(length=2000), autoincrement=False, nullable=True),
    sa.Column('iconPath', sa.VARCHAR(length=200), autoincrement=False, nullable=True),
    sa.Column('routePath', sa.VARCHAR(length=200), autoincrement=False, nullable=True),
    sa.Column('routeParamJson', sa.VARCHAR(length=200), autoincrement=False, nullable=True),
    sa.Column('onDeliveredPayload', postgresql.BYTEA(), autoincrement=False, nullable=True),
    sa.Column('onCompletedPayload', postgresql.BYTEA(), autoincrement=False, nullable=True),
    sa.Column('onDeletedPayload', postgresql.BYTEA(), autoincrement=False, nullable=True),
    sa.Column('autoComplete', sa.INTEGER(), server_default=sa.text('0'), autoincrement=False, nullable=False),
    sa.Column('autoDelete', sa.INTEGER(), server_default=sa.text('0'), autoincrement=False, nullable=False),
    sa.Column('stateFlags', sa.INTEGER(), server_default=sa.text('0'), autoincrement=False, nullable=False),
    sa.Column('notificationRequiredFlags', sa.INTEGER(), server_default=sa.text('0'), autoincrement=False, nullable=False),
    sa.Column('notificationSentFlags', sa.INTEGER(), server_default=sa.text('0'), autoincrement=False, nullable=False),
    sa.Column('displayAs', sa.INTEGER(), server_default=sa.text('0'), autoincrement=False, nullable=False),
    sa.Column('autoDeleteDateTime', postgresql.TIMESTAMP(), autoincrement=False, nullable=True),
    sa.Column('displayPriority', sa.INTEGER(), server_default=sa.text('0'), autoincrement=False, nullable=False),
    sa.Column('onDialogConfirmPayload', postgresql.BYTEA(), autoincrement=False, nullable=True),
    sa.PrimaryKeyConstraint('id', name='Task_pkey'),
    sa.UniqueConstraint('uniqueId', name='Task_uniqueId_key'),
    schema='pl_inbox',
    postgresql_ignore_search_path=False
    )
    op.create_table('Setting',
    sa.Column('id', sa.INTEGER(), server_default=sa.text('nextval(\'pl_inbox."Setting_id_seq"\'::regclass)'), nullable=False),
    sa.Column('name', sa.VARCHAR(), autoincrement=False, nullable=True),
    sa.PrimaryKeyConstraint('id', name='Setting_pkey'),
    schema='pl_inbox',
    postgresql_ignore_search_path=False
    )
    op.create_table('Activity',
    sa.Column('id', sa.INTEGER(), server_default=sa.text('nextval(\'pl_inbox."Activity_id_seq"\'::regclass)'), nullable=False),
    sa.Column('uniqueId', sa.VARCHAR(length=100), autoincrement=False, nullable=False),
    sa.Column('userId', sa.VARCHAR(length=50), autoincrement=False, nullable=False),
    sa.Column('dateTime', postgresql.TIMESTAMP(), autoincrement=False, nullable=False),
    sa.Column('title', sa.VARCHAR(length=100), autoincrement=False, nullable=False),
    sa.Column('description', sa.VARCHAR(length=2000), autoincrement=False, nullable=True),
    sa.Column('iconPath', sa.VARCHAR(length=200), autoincrement=False, nullable=True),
    sa.Column('routePath', sa.VARCHAR(length=200), autoincrement=False, nullable=True),
    sa.Column('routeParamJson', sa.VARCHAR(length=200), autoincrement=False, nullable=True),
    sa.Column('autoDeleteDateTime', postgresql.TIMESTAMP(), autoincrement=False, nullable=False),
    sa.PrimaryKeyConstraint('id', name='Activity_pkey'),
    sa.UniqueConstraint('uniqueId', name='Activity_uniqueId_key'),
    schema='pl_inbox'
    )
    op.create_table('TaskAction',
    sa.Column('id', sa.INTEGER(), server_default=sa.text('nextval(\'pl_inbox."TaskAction_id_seq"\'::regclass)'), nullable=False),
    sa.Column('taskId', sa.INTEGER(), autoincrement=False, nullable=False),
    sa.Column('title', sa.VARCHAR(length=50), autoincrement=False, nullable=True),
    sa.Column('confirmMessage', sa.VARCHAR(length=200), autoincrement=False, nullable=True),
    sa.Column('onActionPayload', postgresql.BYTEA(), autoincrement=False, nullable=True),
    sa.ForeignKeyConstraint(['taskId'], ['pl_inbox.Task.id'], name='TaskAction_taskId_fkey', ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id', name='TaskAction_pkey'),
    schema='pl_inbox'
    )
    op.create_table('SettingProperty',
    sa.Column('id', sa.INTEGER(), server_default=sa.text('nextval(\'pl_inbox."SettingProperty_id_seq"\'::regclass)'), nullable=False),
    sa.Column('settingId', sa.INTEGER(), autoincrement=False, nullable=False),
    sa.Column('key', sa.VARCHAR(length=50), autoincrement=False, nullable=False),
    sa.Column('type', sa.VARCHAR(length=16), autoincrement=False, nullable=True),
    sa.Column('int_value', sa.INTEGER(), autoincrement=False, nullable=True),
    sa.Column('char_value', sa.VARCHAR(), autoincrement=False, nullable=True),
    sa.Column('boolean_value', sa.BOOLEAN(), autoincrement=False, nullable=True),
    sa.ForeignKeyConstraint(['settingId'], ['pl_inbox.Setting.id'], name='SettingProperty_settingId_fkey'),
    sa.PrimaryKeyConstraint('id', name='SettingProperty_pkey'),
    schema='pl_inbox'
    )
    # ### end Alembic commands ###
